      program rmca
      character*4 version
      parameter (version='2.11')
*
************************************************************************
*
*     A general purpose RMC code used for fitting to diffraction
*     data or g(r)'s and incorporating coordination number constraints.
*
*     For more details see the documentation
*
*     This code written by Malcolm Howe (MITCHELL@UK.AC.OXFORD.VAX)
*     Last altered 26th May 1992
*
************************************************************************
*
      parameter (max_atoms =10000)
      parameter (max_expt  =    3)
      parameter (max_q     =  600)
      parameter (max_r     =  550)
      parameter (max_cc    =    4)
      parameter (max_types =    5)
      parameter (max_par   =max_types*(max_types+1)/2)
*
      real      atoms(3,max_atoms)
      real      q(max_q),sexpt(max_q,max_expt)
      real      coeff(max_q,max_expt,max_par) 
      real      gnorm(max_r)
      real      gpar(max_r,max_par),gtot(max_r,max_expt)
      real      sqr(max_r,max_q)
      real      rcut(max_par),delta(max_types)
      real      coeff0(max_expt,max_par)
      real      sigma(max_expt),chisq(max_expt)
      real      sum_s(max_expt),sum_ss(max_expt),sum_fs(max_expt)
      real      sum_f(max_expt),sum_ff(max_expt)
      real      factor(max_expt),constant(max_expt)
      real      rcoord(max_cc),rcoord_0(max_cc),coordfrac(max_cc)
      real      sigmac(max_cc),cfnew(max_cc)
      real      move(6)
      real      vectors(3,3),metric(3,3)

      integer   nq1(max_expt),nq2(max_expt)
      integer   ni(max_types),ncum(max_types),nxn(max_par)
      integer   histogram(max_r,max_par),dhistogram(max_r,max_par)
      integer   typec(max_cc),typen(max_cc),coordno(max_cc)
      integer   typec_0(max_cc),typen_0(max_cc)
      integer   nneigh(max_cc,max_atoms),dnneigh(max_cc,max_atoms)
      integer   nneighi(max_cc)
      logical   truncated,renorm,offset,fittos
      logical   first,rerun,looping,acceptable
      logical   infile(max_expt),transpar,exists,calchis
      character*80 title,file,sqfile(max_expt)
      common /space/ sqr,histogram,dhistogram
      data      rerun /.true./
*
*-------------------------------------------------------------------------
*     Initialisation
*
      call rmc_init(iseed,time_start)
      call rmc_datafile(file)
      call open_read(4,file,'.dat')
*
*-------------------------------------------------------------------------
*     Obtain name of files to be used
*
      read (4,'(a)') file
      lfile=len(file)
      do while (file(lfile:lfile).eq.' ')
         lfile=lfile-1
      enddo
      write (*,100) version,file(1:lfile)
*
*-------------------------------------------------------------------------
*     If possible read the intermediate (histogram) file
*     otherwise read the configuration file
*
      calchis=.false.
      inquire(file=file(1:lfile)//'.his',exist=exists)
      if (exists) then
         write (*,*) 'Reading intermediate (histogram) file ',
     -    file(1:lfile),'.his'
         call open_read(3,file,'.his')
         call read_his(3,title,ngen,ntried,nacc,nsaved,
     -    truncated,vectors,rho_0,n,ntypes,max_types,ni,atoms,
     -    nr,max_r,dr_0,histogram,ncoord_0,max_cc,typec_0,typen_0,
     -    rcoord_0,nneigh,calchis)
         close(3)
         if (calchis) then
            write (*,*) 'ERROR READING FILE'
            write (*,*) 'Configuration file will be read instead'
            write (*,*)
         endif
      else
         calchis=.true.
      endif
*
*     Otherwise read the configuration file
*
      if (calchis) then
         write (*,*) 'Reading configuration file ',
     -    file(1:lfile),'.cfg'
         call open_read(3,file,'.cfg')
         call read_config(title,ngen,ntried,nacc,nsaved,max_atoms,
     -    max_types,truncated,vectors,n,ntypes,ni,atoms,3)
         close(3)
      endif
      npar=ntypes*(ntypes+1)/2
      write (*,110) n,ntypes
*
*-------------------------------------------------------------------------
*     Read remaining data for programme
*
      call rmca_readdata(4,max_expt,ntypes,npar,title,rho,rcut,ncoord,
     - max_cc,typec,typen,rcoord,coordno,coordfrac,sigmac,fittos,nexpt,
     - sigma,delta,dr,renorm,offset,ncoll,iprint,
     - timelim,timesav,sqfile,constant,nq1,nq2,infile,coeff0)
      close (4)
*
*-------------------------------------------------------------------------
*     Read experimental data
*
      if (nexpt.eq.0) then
         nq=100
         do iq=1,nq
            q(iq)=real(iq)/10.
         enddo
      else
         call rmca_readsq(sqfile,nexpt,max_expt,constant,max_q,
     -    nq1,nq2,nq,q,sexpt,sum_f,sum_ff,infile,npar,coeff0,coeff)
         if (.not.fittos) dr=q(2)-q(1)
      endif
*
*-------------------------------------------------------------------------
*     We must recalculate the histogram if we have changed 
*     the density, the r spacing, or the coordination constraints
*
      if (.not.calchis) then
         if (rho.ne.rho_0) calchis=.true.
         if (dr.ne.dr_0) calchis=.true.
         if (ncoord.ne.ncoord_0.and.ncoord.gt.0) then
            calchis=.true.
         else
            do i=1,ncoord
               if (typec_0(i).ne.typec(i)) calchis=.true.
               if (typen_0(i).ne.typen(i)) calchis=.true.
               if (rcoord_0(i).ne.rcoord(i)) calchis=.true.
            enddo
         endif
         if (calchis) write (*,115)
      endif
*
*-------------------------------------------------------------------------
*     Find volume of configuration cell
*
      call rmc_volume(vectors,truncated,volume)
*
*-------------------------------------------------------------------------
*     If necessary rescale configuration cell to required density
*     and calculate the number of histogram points
*
      if (calchis) then
         call rmc_scale(volume,n,rho,vectors)
         call rmc_cellsize(vectors,truncated,volume,d)
         nr=int(d/dr)
      endif
      if (nr.gt.max_r) then
         write (*,*) 
         write (*,*) 'WARNING: r array is too small; ',
     -    'the number or r points has been reduced'
         write (*,*)
         nr=max_r
      endif
      if (.not.fittos) then
         do iexpt=1,nexpt
            nq2(iexpt)=min(nq2(iexpt),nr)
         enddo
         nq=min(nq,nr)
      endif
      write (*,120) nr,dr,nr*dr
*
*-------------------------------------------------------------------------
*     Calculate metric matrix
*
      call rmc_metric(vectors,metric)
*
*-------------------------------------------------------------------------
*     Calculate arrays needed to convert histograms to g(r)
*
      call rmc_gnorm(ntypes,ni,nxn,nr,dr,volume,gnorm)
      ncum(1)=ni(1)
      do itype=2,ntypes
         ncum(itype)=ncum(itype-1)+ni(itype)
      enddo
*
*-------------------------------------------------------------------------
*     Decide whether to transform partials or totals
*
      transpar=.false.
      do iexpt=1,nexpt
         transpar=transpar.or.infile(iexpt)
      enddo         
      if (nexpt.gt.npar) transpar=.true. 
*
*-------------------------------------------------------------------------
*     If necessary calculate partial g(r) histograms and 
*     coordination numbers
*
      if (calchis) then
         write (*,*)
         write (*,*) 'Calculating g(r) histogram'
         call rmc_calhist(n,atoms,ntypes,truncated,metric,
     -    ncum,dr,nr,max_r,npar,histogram,ncoord,max_cc,typec,typen,
     -    rcoord,nneigh)
         write (*,*)
         write (*,*) 'Writing g(r) histogram to disk'
         write (*,*) 
     -     '*** DO NOT INTERRUPT UNTIL SAVING IS COMPLETED ***'
         call open_write(3,file,'.his')
         call write_his(3,title,ngen,ntried,nacc,nsaved,
     -    truncated,vectors,rho,n,ntypes,max_types,ni,atoms,
     -    nr,max_r,dr,histogram,ncoord,max_cc,typec,typen,
     -    rcoord,nneigh)
         close(3)
         write (*,*) 'Saving has been completed'
      endif
*
*-------------------------------------------------------------------------
*     Check whether the configuration satisfies the cut-offs
*
      do itype=1,ntypes
         do jtype=itype,ntypes
            ic=(itype-1)*(2*ntypes-itype)/2+jtype
            do ir=1,nint(rcut(ic)/dr)-1
               if (histogram(ir,ic).gt.0) 
     -          write (*,130) histogram(ir,ic),itype,jtype,real(ir)*dr
            enddo
         enddo
      enddo
*
*-------------------------------------------------------------------------
*     Calculate the sin(Qr) table
*
      call rmc_sinqr(dr,nr,max_r,q,1,nq,rho,sqr)
*
*-------------------------------------------------------------------------
*-------------------------------------------------------------------------
*     This is where the loop begins. 
*
      save_time=min(timelim,timesav)
      time_of_loop=0.
      call rmc_time(time_start,timer_on)
      timer_on=abs(timer_on)
      ngen0=ngen
      lastprint=ngen-mod(ngen,iprint)
      first=.true.
      acceptable=.true.
      looping=.true.
      ntried=ntried-1
      do while (looping)
*
*-------------------------------------------------------------------------
*        Set change in histogram to zero
*
         do ic=1,npar
            do ir=1,nr
               dhistogram(ir,ic)=0
            enddo
         enddo
         do icc=1,ncoord
            do i=1,n
               dnneigh(icc,i)=0
            enddo
            nneighi(icc)=0
         enddo
         chisq1=0.
*
         if (.not.first) then
*-------------------------------------------------------------------------
*           Generate a random move
*
            call rmc_genmove(iseed,n,ncum,delta,metric,imove,move)
            call rmc_newpos(move,imove,1,atoms,truncated)
            ngen=mod(ngen+1,1000000)
            acceptable=.true.
*
*-------------------------------------------------------------------------
*           Calculate change in histogram 
*
            call rmc_changehist(1,n,atoms,ntypes,truncated,
     -       metric,ncum,dr,nr,max_r,rcut,
     -       imove,move,dhistogram,max_cc,ncoord,typec,
     -       typen,rcoord,dnneigh,nneighi,acceptable)
            do icc=1,ncoord
               dnneigh(icc,imove)=nneighi(icc)
            enddo
         endif
*
*----------------------------------------------------------------------
         if (acceptable) ntried=mod(ntried+1,1000000)
         if (acceptable.and.nexpt.gt.0) then
*
*----------------------------------------------------------------------
*           Calculate new partial g(r)'s 
* 
            do ir=1,nr
               do ic=1,npar
                  gpar(ir,ic)=real(histogram(ir,ic)+dhistogram(ir,ic))/
     -             (gnorm(ir)*real(nxn(ic)))
               enddo
               do iexpt=1,nexpt
                  gtot(ir,iexpt)=0.
                  do ic=1,npar
                     gtot(ir,iexpt)=gtot(ir,iexpt)+coeff(1,iexpt,ic)*
     -                (gpar(ir,ic)-1.)
                  enddo
               enddo
            enddo
*
*----------------------------------------------------------------------
*           Calculate sums for chi-squared
* 
            if (fittos) then
               if (transpar) then
                  call rmc_sumspar(max_r,max_q,max_expt,nexpt,npar,nq1,
     -             nq2,1,nq,nr,gpar,sqr,coeff,sexpt,sum_s,sum_ss,sum_fs)
               else
                  call rmc_sumstot(max_r,max_q,nexpt,nq1,nq2,1,nq,nr,
     -             gtot,sqr,sexpt,sum_s,sum_ss,sum_fs)
               endif
            else
               call rmc_sumg(max_r,max_q,nexpt,nq1,nq2,1,nq,gtot,sexpt,
     -          sum_s,sum_ss,sum_fs)
            endif
*
*----------------------------------------------------------------------
*           Calculate chi-squared
*
            call rmc_chisq(nexpt,nq1,nq2,renorm,offset,
     -       factor,constant,sum_s,sum_fs,sum_ss,sum_f,sum_ff,
     -       sigma,chisq,chisq1)
         endif
         if (ncoord.gt.0) call rmc_coord(ni,ncum,ncoord,
     -    max_cc,typec,coordno,nneigh,dnneigh,
     -    sigmac,cfnew,coordfrac,chisq1)
*
*----------------------------------------------------------------------
*        Decide whether to accept move
*
         if (first) then
            acceptable=.false.
            chisq0=chisq1
         else
            prat=(chisq0-chisq1)/2.
            if (prat.lt.1) prat=exp(prat)
            if (prat.lt.ran(iseed)) acceptable=.false.
         endif
         if (acceptable) nacc=mod(nacc+1,1000000)
*
*----------------------------------------------------------------------
*        Write summary
*
         if (first.or.(abs(ngen-lastprint).ge.iprint.and.acceptable)) 
     -    then
            call rmca_summary(nacc,ngen,ntried,nexpt,nq1,nq2,
     -       factor,constant,chisq,cfnew,coordfrac,ncoord,chisq1)
            lastprint=ngen-mod(ngen,iprint)
         endif
*
         if (acceptable) then

*----------------------------------------------------------------------
*           Make move
*
            call rmc_makemove(atoms,n,imove,move,nr,max_r,
     -       npar,histogram,dhistogram,ncoord,max_cc,nneigh,dnneigh)
            chisq0=chisq1

*----------------------------------------------------------------------
*           if collecting then save every nth good configuration
*
            if (ncoll.gt.0.and.nsaved.lt.ncoll.and.nacc.ge.n) then
               write (*,*)
               write (*,*) 'Saving configuration to disk'
               write (*,*)
     -          '*** DO NOT INTERRUPT UNTIL SAVING IS COMPLETED ***'
               call open_write(3,file,'.sav')
               call write_config(title,ngen,ntried,nacc,nsaved,
     -          truncated,vectors,n,ntypes,ni,atoms,3)
               close(3)
               nsaved=nsaved+1
               write (*,*) 'Saving has been completed'
               write (*,*)
               write (*,'(a,i3,a)') 'Configuration number ',nsaved,
     -          ' has been saved'
               write (*,*)
               nacc=0
               ngen=0
               ntried=0
               if (nsaved.ge.ncoll) rerun=.false.
            endif
         endif
*
*----------------------------------------------------------------------
*        Check time
*
         call rmc_time(time_start,time_used)
         if (time_used.lt.0.or.time_used.ge.save_time) then
*
*----------------------------------------------------------------------
*           Calculate and display timing details
*
            time_of_loop=time_of_loop+(abs(time_used)-timer_on)
            write (*,*)
            write (*,*) '  Total time in loop so far :',
     -       time_of_loop*60,'s'
            write (*,*) '  Time per generated move is:',
     -       time_of_loop*60000/real(max(ngen-ngen0,3)),'ms'
*
*----------------------------------------------------------------------
*           Write out the configuration, the g(r) histogram
*           and the output file
*
            write (*,*)
            write (*,*) 'Saving configuration and results to disk'
            write (*,*) 
     -       '*** DO NOT INTERRUPT UNTIL SAVING IS COMPLETED ***'
            call open_write(3,file,'.cfg')
            call write_config(title,ngen,ntried,nacc,nsaved,
     -       truncated,vectors,n,ntypes,ni,atoms,3)
            close(3)
*
            call open_write(3,file,'.his')
            call write_his(3,title,ngen,ntried,nacc,nsaved,
     -       truncated,vectors,rho,n,ntypes,max_types,ni,atoms,
     -       nr,max_r,dr,histogram,ncoord,max_cc,typec,typen,
     -       rcoord,nneigh)
            close(3)
*
            call open_write(3,file,'.out')
            call rmca_output(version,title,max_r,max_q,max_expt,
     -       nr,nq1,nq2,nq,nexpt,npar,dr,histogram,gnorm,nxn,gpar,gtot,
     -       q,sexpt,sqr,fittos,factor,constant,coeff,3)
            close(3)
            write (*,*) 'Saving has been completed'
            write (*,*)
            save_time=save_time+timesav
            call rmc_time(time_start,timer_on)
         endif
*
*----------------------------------------------------------------------
*        Decide whether to continue programme
*
         if ((time_used.lt.0.).or.(time_used.ge.timelim)
     -    .or..not.rerun) looping=.false.
         first=.false.
      enddo
*
      call rmc_rerun(rerun)
      stop
100   format(/1x,'======================'/
     -        1x,' RMCA version ',a/
     -        1x,'======================'//
     -        1x,'Using files : ',a/)
110   format(/1x,'Configuration contains ',i5,' atoms of ',i1,' types'/) 
115   format(1x,'Parameters have been changed: ',
     - 'Histogram will be recalculated')
120   format(/' Using ',i4,' r points spaced at ',f5.3,
     - 'A up to ',f6.3,'A'/)
130   format(' WARNING: ',i3,' pairs cause g',2i1,' to be ',
     - 'non-zero at ',f4.2,'A')
      end



      subroutine rmca_readdata(ichan,max_expt,ntypes,npar,title,rho,
     - rcut,ncoord,max_cc,typec,typen,rcoord,coordno,coordfrac,sigmac,
     - fittos,nexpt,sigma,delta,dr,renorm,offset,ncoll,iprint,
     - timelim,timesav,sqfile,constant,nq1,nq2,infile,coeff0)
      real          rcut(*),sigma(*),constant(*)
      real          coeff0(max_expt,*)
      real          rcoord(*),coordfrac(*),delta(*),sigmac(*)
      integer       nq1(max_expt),nq2(max_expt)
      integer       typec(*),typen(*),coordno(*)
      character*(*) title,sqfile(*)
      logical fittos,offset,renorm,infile(*)
      read (ichan,'(a)') title
      read (ichan,*) rho
      read (ichan,*) (rcut(ic),ic=1,npar)
      read (ichan,*) ncoord
      if (ncoord.gt.max_cc) then
         write (*,*) ncoord,' coordination constraints are given'
         write (*,*) 'Arrays are only large enough for ',max_cc
         stop
      endif
      do i=1,ncoord
         read (ichan,*) typec(i),typen(i),rcoord(i),coordno(i),
     -    coordfrac(i),sigmac(i)
      enddo
      read (ichan,*) fittos
      read (ichan,*) nexpt
      if (nexpt.gt.max_expt) then
         write (*,*) nexpt,' experimental data sets are given'
         write (*,*) 'Arrays are only large enough for ',max_expt
         stop
      endif
      if (nexpt.gt.0) read (ichan,*) (sigma(iexpt),iexpt=1,nexpt)
      read (ichan,*) (delta(i),i=1,ntypes)
      read (ichan,*) dr
      read (ichan,*) renorm
      read (ichan,*) offset
      read (ichan,*) ncoll
      read (ichan,*) iprint
      read (ichan,*) timelim,timesav
      do iexpt=1,nexpt
         read (ichan,'(a)') sqfile(iexpt)
         read (ichan,*) nq1(iexpt),nq2(iexpt)
         read (ichan,*) constant(iexpt)
         read (ichan,*) infile(iexpt)
         if (.not.infile(iexpt)) read (ichan,*) 
     -    (coeff0(iexpt,ic),ic=1,npar)
      enddo
      if (nexpt.eq.0) then
         fittos=.false.
         renorm=.false.
         offset=.false.
      endif
*
*     Now list to terminal
*
      write (*,100) title
      if (nexpt.eq.0) then
         write (*,110)
      else
         if (fittos) then
            write (*,120) nexpt
         else
            write (*,130) nexpt
         endif
         do iexpt=1,nexpt
            write (*,140) iexpt,sqfile(iexpt),nq1(iexpt),nq2(iexpt),
     -       constant(iexpt),sigma(iexpt)
            if (infile(iexpt)) then
               write (*,150)
            else
               write (*,160) 
     -          (coeff0(iexpt,ic),ic=1,ntypes*(ntypes+1)/2)
            endif
         enddo
      endif
      write (*,170)
      do i=1,ntypes
         do j=i,ntypes
             write (*,180) i,j,rcut((i-1)*(2*ntypes-i)/2+j)
         enddo
      enddo
      write (*,190) ncoord
      do i=1,ncoord
         write (*,200) typen(i),rcoord(i),typec(i),
     -    coordno(i),coordfrac(i)*100.,sigmac(i)
      enddo
      write (*,210) rho
      do i=1,ntypes
         write (*,220) delta(i),i
      enddo
      write (*,230) ncoll
      if (renorm) write (*,240)
      if (offset) write (*,250)
      write (*,260) iprint,timelim,timesav
      return
100   format(/1x,'Title of run: ',a65/)
110   format(1x,'No S(Q) or g(r) constraints')
120   format(1x,'Fitting to ',i1,' F(Q)''s')
130   format(1x,'Fitting to ',i1,' g(r)''s')
140   format(/1x,'Data set ',i1,' is read from file ',a50/
     - 1x,'Using points ',i4,' to ',i4,' after subtraction of ',f6.3/
     - 1x,'Standard deviation is ',f7.4)
150   format(1x,'Coefficients of partials structure factors will be ',
     - 'read from file')
160   format(1x,'Coefficients of partials structure factors are ',
     - 4(f7.5,1x)/1000(47x,4(f7.5,1x)/))
170   format(/' Cut offs in partial g(r)''s are at:')
180   format(10x,i1,'-',i1,5x,f6.3,' A')
190   format(/' There are ',i2,' coordination constraints')
200   format('    Constraint on number of atoms of type ',i2,' within ',
     - f6.2,'A of ones of type ',i2,' is ',i2,' at ',f5.1,
     - '% with sigma=',f7.4) 
210   format(/' Number density is ',f6.4,' atoms per cubic Angstrom'/)
220   format(' Maximum change in any coordinate is ',f6.4,
     - 'A for particles of type ',i2)
230   format(/' No. of configs to save    : ',i3/)
240   format(' Renormalising experimental data')
250   format(' Rezeroing experimental data')
260   format(/' Writing summary every ',i4,' generated moves'/
     - ' Programme will run for ',f7.1,' minutes saving every ',
     - f5.1,' minutes'/)
      end




      subroutine rmca_readsq(sqfile,nexpt,max_expt,constant,max_q,
     - nq1,nq2,nq,q,sexpt,sum_f,sum_ff,infile,npar,coeff0,coeff)
      real          q(*),constant(*),sexpt(max_q,*)
      real          coeff0(max_expt,*),coeff(max_q,max_expt,*)
      real          sum_f(*),sum_ff(*)
      integer       nq1(*),nq2(*)
      logical       infile(*)
      character*(*) sqfile(*)
*
*     Read experimental data. If there is a q=0 point remove it.
*
      do iexpt=1,nexpt
         call open_read(3,sqfile(iexpt),'.dat')
         read (3,*) nq
         read (3,*) 
         if (iexpt.eq.1) then
            nq_1=nq
         elseif (nq.ne.nq_1) then
            write (*,*) 'ERROR: Experimental data sets must contain ',
     -       'the same number of points'
            stop
         endif
*
         if (nq.gt.max_q) then
            write (*,*) 'Experimental data contains ',nq, 'points'
            write (*,*) 'Arrays are only large enough for ',max_q
            stop
         endif
         nq2(iexpt)=min(nq,nq2(iexpt))
*
         sum_f(iexpt)=0.
         sum_ff(iexpt)=0.
         do iq=1,nq
            if (infile(iexpt)) then
               read (3,*) q(iq),sexpt(iq,iexpt),
     -          (coeff(iq,iexpt,ic),ic=1,npar)
            else
               read (3,*) q(iq),sexpt(iq,iexpt)
               do ic=1,npar
                  coeff(iq,iexpt,ic)=coeff0(iexpt,ic)
               enddo
            endif             
            if (q(iq).le.0) nq1(iexpt)=max(iq+1,nq1(iexpt))
            sexpt(iq,iexpt)=sexpt(iq,iexpt)-constant(iexpt)
            if (iq.ge.nq1(iexpt).and.iq.le.nq2(iexpt)) then
               sum_f(iexpt)=sum_f(iexpt)+sexpt(iq,iexpt)
               sum_ff(iexpt)=sum_ff(iexpt)+sexpt(iq,iexpt)**2
            endif
         enddo
         close(3)
      enddo
      return
      end




      subroutine rmca_summary(nacc,ngen,ntried,nexpt,nq1,nq2,
     - factor,constant,chisq,cfnew,coordfrac,ncoord,chisq1)
      real    factor(*),constant(*),chisq(*),cfnew(*),coordfrac(*)
      integer nq1(*),nq2(*)
      ndof=ncoord
      do iexpt=1,nexpt
         ndof=ndof+nq2(iexpt)-nq1(iexpt)+1
      enddo
      if (nexpt.eq.0.and.ncoord.eq.0) then
         write (*,'(1x,i6,a,i6,a)')
     -   nacc,' moves accepted out of ',ngen,' generated'
      else
         write (*,*)
         write (*,'(1x,i6,a,i6,a,i6,a,g10.4)') nacc,' moves acc. ',
     -    ngen,' gen. and ',ntried,
     - ' tested; Chi-squared/d.o.f.=',chisq1/real(ndof)
      endif
      do iexpt=1,nexpt
         qn=nq2(iexpt)-nq1(iexpt)+1
         write (*,'(1x,a,i2,a,f6.4,a,f7.4,a,g10.4)')
     -    '     Expt ',iexpt,': Renorm.=',1./factor(iexpt),
     -    '; Constant=',constant(iexpt),'; Chi**2/nq=',chisq(iexpt)/qn
      enddo
      do icc=1,ncoord
         write (*,'(1x,a,i2,a,f6.2,a,f6.2,a)')
     -    '     Coordination constraint ',icc,': Fraction=',
     -    cfnew(icc)*100.,'%; target=',coordfrac(icc)*100.,'%'
      enddo
      return
      end




      subroutine rmca_output(version,title,max_r,max_q,max_expt,
     - nr,nq1,nq2,nq,nexpt,npar,dr,histogram,gnorm,nxn,gpar,gtot,
     - q,sexpt,sqr,fittos,factor,constant,coeff,iprint)
      real          q(*),sexpt(max_q,*),gnorm(*)
      real          gpar(max_r,*),gtot(max_r,*),factor(*),constant(*)
      real          coeff(max_q,max_expt,*),sqr(max_r,*)
      real          s(100)
      integer       histogram(max_r,*),nxn(*),nq1(*),nq2(*)
      logical       fittos
      character*(*) version,title
      write (iprint,*) 'Results from RMCA version ',version
      write (iprint,*) 'TITLE'
      write (iprint,*) title
      write (iprint,*) nexpt,' experiments;',npar,' partials'
      write (iprint,*) 'Partial g(r)''s'
      write (iprint,*) ' r, g(r)'
      write (iprint,*) 'PLOTS'
      write (iprint,*) nr,',',npar
      do ir=1,nr
         do ic=1,npar
            gpar(ir,ic)=real(histogram(ir,ic))/
     -       (gnorm(ir)*real(nxn(ic)))
         enddo
         write (iprint,*) real(ir)*dr,(gpar(ir,ic),ic=1,npar)
      enddo
      if (nexpt.gt.0) then
         write (iprint,*) 'ENDGROUP'
         write (iprint,*) 'Total g(r)''s'
         do ir=1,nr
            do iexpt=1,nexpt
               gtot(ir,iexpt)=0.
               do ic=1,npar
                  gtot(ir,iexpt)=gtot(ir,iexpt)+coeff(1,iexpt,ic)*
     -             (gpar(ir,ic)-1.)
               enddo
            enddo
         enddo
         if (fittos) then
            write (iprint,*) ' r, g(r)'
            write (iprint,*) 'PLOTS'
            write (iprint,*) nr,',',nexpt
            do ir=1,nr
               write (iprint,*) real(ir)*dr,(gtot(ir,iexpt),
     -          iexpt=1,nexpt)
            enddo
         else
            write (iprint,*) 'Comparison with input data'
            do iexpt=1,nexpt
               write (iprint,*) ' r, g(r) (RMC), g(r) (Expt)'
               write (iprint,*) 'CURVES'
               write (iprint,*) nq2(iexpt)-nq1(iexpt)+1,',',2
               do iq=nq1(iexpt),nq2(iexpt)
                  write (iprint,*) q(iq),gtot(iq,iexpt),
     -             (sexpt(iq,iexpt)-constant(iexpt))/factor(iexpt)
               enddo
            enddo
         endif
      endif
      write (iprint,*) 'ENDGROUP'
      write (iprint,*) 'Partial S(Q)''s'
      write (iprint,*) ' Q, S(Q)'
      write (iprint,*) 'PLOTS'
      write (iprint,*) nq,',',npar
      do iq=1,nq
         do ic=1,npar
            s(ic)=0.
            do ir=1,nr
               s(ic)=s(ic)+(gpar(ir,ic)-1.)*sqr(ir,iq)
            enddo
         enddo
         write (iprint,*) q(iq),(s(ic),ic=1,npar)
      enddo
      if (nexpt.gt.0) then
         write (iprint,*) 'ENDGROUP'
         write (iprint,*) 'Total S(Q)''s'
         if (fittos) then
            write (iprint,*) 'Comparison with input data'
            do iexpt=1,nexpt
               write (iprint,*) ' Q, S(Q) (RMC), S(Q) (Expt)'
               write (iprint,*) 'CURVES'
               write (iprint,*) nq2(iexpt)-nq1(iexpt)+1,',',2
               do iq=nq1(iexpt),nq2(iexpt)
                  snew=0.
                  do ic=1,npar
                     spar=0.
                     do ir=1,nr
                        spar=spar+(gpar(ir,ic)-1.)*sqr(ir,iq)
                     enddo
                     snew=snew+spar*coeff(iq,iexpt,ic)
                  enddo
                  write (iprint,*) q(iq),snew,
     -             (sexpt(iq,iexpt)-constant(iexpt))/factor(iexpt)
               enddo
            enddo
         else
            write (iprint,*) ' Q, S(Q)'
            write (iprint,*) 'PLOTS'
            write (iprint,*) nq,',',nexpt
            do iq=1,nq
               do iexpt=1,nexpt
                  s(iexpt)=0.
                  do ir=1,nr
                     s(iexpt)=s(iexpt)+gtot(ir,iexpt)*sqr(ir,iq)
                  enddo
               enddo
               write (iprint,*) q(iq),(s(iexpt),iexpt=1,nexpt)
            enddo
         endif
      endif
      return
      end




      subroutine write_his(ichan,title,ngen,ntried,nacc,nsaved,
     - truncated,vectors,rho,n,ntypes,max_types,ni,atoms,
     - nr,max_r,dr,histogram,
     - ncoord,max_cc,typec,typen,rcoord,nneigh)
      real atoms(3,*),vectors(3,3),rcoord(max_cc)
      integer ni(max_types),histogram(max_r,*)
      integer typec(max_cc),typen(max_cc),nneigh(max_cc,*)
      logical truncated
      character*(*) title
      character*8 real_to_hex,hex(9)
      external real_to_hex
*
      write (ichan,'(1x,a)') 'RMCA intermediate (histogram) file'
      write (ichan,'(1x,a)') title
      write (ichan,*) ngen,ntried,nacc,nsaved,truncated
      do i=1,3
         do j=1,3
            hex((i-1)*3+j)=real_to_hex(vectors(j,i))
         enddo
      enddo
      write (ichan,10) (hex(i),i=1,9)
10    format(1x,9a8)
      hex(1)=real_to_hex(rho)
      write (ichan,20) hex(1),n,ntypes
20    format(1x,a8,1x,2i10)
      write (ichan,*) (ni(i),i=1,ntypes)
      do i=1,n
         hex(1)=real_to_hex(atoms(1,i))
         hex(2)=real_to_hex(atoms(2,i))
         hex(3)=real_to_hex(atoms(3,i))
         write (ichan,30) (hex(j),j=1,3)
      enddo
30    format(3(1x,a8))
      hex(1)=real_to_hex(dr)
      write (ichan,40) nr,hex(1)
40    format(i10,1x,a8)
      do i=1,ntypes*(ntypes+1)/2
         write (ichan,50) (histogram(j,i),j=1,nr)
      enddo
50    format(10(1x,i7))
      write (ichan,*) ncoord
      do i=1,ncoord
         hex(1)=real_to_hex(rcoord(i))
         write (ichan,60) typec(i),typen(i),hex(1)
         write (ichan,50) (nneigh(i,j),j=1,n)
      enddo
60    format(2i10,1x,a8)
      return
      end




      subroutine read_his(ichan,title,ngen,ntried,nacc,nsaved,
     - truncated,vectors,rho,n,ntypes,max_types,ni,atoms,
     - nr,max_r,dr,histogram,
     - ncoord,max_cc,typec,typen,rcoord,nneigh,error)
      real atoms(3,*),vectors(3,3),rcoord(max_cc)
      integer ni(max_types),histogram(max_r,*)
      integer typec(max_cc),typen(max_cc),nneigh(max_cc,*)
      logical truncated,error
      character*(*) title
      character*8 hex(9)
*
      error=.false.
      read (ichan,*,end=100,err=100) 
      read (ichan,'(1x,a)',end=100,err=100) title
      read (ichan,*,end=100,err=100) ngen,ntried,nacc,nsaved,truncated
      read (ichan,10,end=100,err=100) (hex(i),i=1,9)
10    format(1x,9a8)
      do i=1,3
         do j=1,3
            vectors(j,i)=hex_to_real(hex(3*(i-1)+j),error)
            if (error) goto 100
         enddo
      enddo
      read (ichan,20,end=100,err=100) hex(1),n,ntypes
20    format(1x,a8,1x,2i10)
      rho=hex_to_real(hex(1),error)
      if (error) goto 100
      read (ichan,*,end=100,err=100) (ni(i),i=1,ntypes)
      do i=1,n
         read (ichan,30,end=100,err=100) (hex(j),j=1,3)
         do j=1,3
            atoms(j,i)=hex_to_real(hex(j),error)
            if (error) goto 100
         enddo
      enddo
30    format(3(1x,a8))
      read (ichan,40,end=100,err=100) nr,hex(1)
40    format(i10,1x,a8)
      dr=hex_to_real(hex(1),error)
      if (error) goto 100
      do i=1,ntypes*(ntypes+1)/2
         read (ichan,*,end=100,err=100) (histogram(j,i),j=1,nr)
      enddo
      read (ichan,*,end=100,err=100) ncoord
      do i=1,ncoord
         read (ichan,60,end=100,err=100) typec(i),typen(i),hex(1)
         rcoord(i)=hex_to_real(hex(1),error)
         if (error) goto 100
         read (ichan,*,end=100,err=100) (nneigh(i,j),j=1,n)
      enddo
      return
60    format(2i10,1x,a8)
100   error=.true.
      return
      end
