      subroutine rmc_volume(vectors,truncated,volume)
      real vectors(3,3)
      logical truncated
*     
*    Finds the volume of the configuration cell
*
      triprod=vectors(1,1)*vectors(2,2)*vectors(3,3)
     -       +vectors(2,1)*vectors(3,2)*vectors(1,3)
     -       +vectors(3,1)*vectors(1,2)*vectors(2,3)
     -       -vectors(3,1)*vectors(2,2)*vectors(1,3)
     -       -vectors(2,1)*vectors(1,2)*vectors(3,3)
     -       -vectors(1,1)*vectors(3,2)*vectors(2,3)
      volume=8.*abs(triprod)
      if (truncated) volume=volume/2.
      return
      end




      subroutine rmc_scale(volume,n,rho,vectors)
      real vectors(3,3)
*     
*     Rescales the configuration cell to get the density correct
*
      f=(real(n)/(rho*volume))**(1./3.)
      do j=1,3
         do i=1,3
            vectors(i,j)=vectors(i,j)*f
         enddo
      enddo
      volume=real(n)/rho
      return
      end



      subroutine rmc_metric(vectors,metric)
      real vectors(3,3),metric(3,3)
*     
*     Calculates the metric matrix used for calculating distances
*
      do j=1,3
         do i=1,3
            metric(i,j)=0.
            do k=1,3
               metric(i,j)=metric(i,j)+vectors(k,i)*vectors(k,j)
            enddo
         enddo
      enddo
      return
      end



      subroutine rmc_cellsize(vectors,truncated,volume,d)
      real vectors(3,3)
      logical truncated
*     
*     Find shortest distance from centre of cell to a face
*
      axb1=vectors(2,1)*vectors(3,2)-vectors(3,1)*vectors(2,2)
      axb2=vectors(3,1)*vectors(1,2)-vectors(1,1)*vectors(3,2)
      axb3=vectors(1,1)*vectors(2,2)-vectors(2,1)*vectors(1,2)
      bxc1=vectors(2,2)*vectors(3,3)-vectors(3,2)*vectors(2,3)
      bxc2=vectors(3,2)*vectors(1,3)-vectors(1,2)*vectors(3,3)
      bxc3=vectors(1,2)*vectors(2,3)-vectors(2,2)*vectors(1,3)
      cxa1=vectors(2,3)*vectors(3,1)-vectors(3,3)*vectors(2,1)
      cxa2=vectors(3,3)*vectors(1,1)-vectors(1,3)*vectors(3,1)
      cxa3=vectors(1,3)*vectors(2,1)-vectors(2,3)*vectors(1,1)
      triprod=volume/8.
      if (truncated) triprod=volume/4.
      d1=triprod/sqrt(axb1**2+axb2**2+axb3**2)
      d2=triprod/sqrt(bxc1**2+bxc2**2+bxc3**2)
      d3=triprod/sqrt(cxa1**2+cxa2**2+cxa3**2)
      d=min(d1,d2,d3)
      if (truncated) then
         d1=1.5*triprod/sqrt(
     -    (axb1+bxc1+cxa1)**2+(axb2+bxc2+cxa2)**2+(axb3+bxc3+cxa3)**2)
         d2=1.5*triprod/sqrt(
     -    (axb1-bxc1+cxa1)**2+(axb2-bxc2+cxa2)**2+(axb3-bxc3+cxa3)**2)
         d3=1.5*triprod/sqrt(
     -    (axb1+bxc1-cxa1)**2+(axb2+bxc2-cxa2)**2+(axb3+bxc3-cxa3)**2)
         d4=1.5*triprod/sqrt(
     -    (axb1-bxc1-cxa1)**2+(axb2-bxc2-cxa2)**2+(axb3-bxc3-cxa3)**2)
         d=min(d,d1,d2,d3,d4)
      endif
      return
      end






      subroutine rmc_calhist(n,atoms,ntypes,truncated,metric,
     - ncum,dr,nr,max_r,npar,histogram,ncoord,max_cc,typec,typen,rcoord,
     - nneigh)
      real atoms(3,*),rcoord(*),metric(3,3)
      integer histogram(max_r,npar)
      integer ncum(*),typec(*),typen(*)
      integer nneigh(max_cc,*)
      integer type1,type2
      logical truncated
*
      do ic=1,npar
         do ir=1,nr
            histogram(ir,ic)=0
         enddo
      enddo
      do i=1,n
         do icc=1,ncoord
            nneigh(icc,i)=0
         enddo
      enddo
      type2=1
      do i=1,n
         if (i.gt.ncum(type2)) type2=type2+1
         type1=1
         do j=1,i-1
            if (j.gt.ncum(type1)) type1=type1+1
            ic=(type1-1)*(2*ntypes-type1)/2+type2
            x=atoms(1,i)-atoms(1,j)+3.
            y=atoms(2,i)-atoms(2,j)+3.
            z=atoms(3,i)-atoms(3,j)+3.
            x=2.*(x/2.-int(x/2.))-1.
            y=2.*(y/2.-int(y/2.))-1.
            z=2.*(z/2.-int(z/2.))-1.
            if (truncated.and.abs(x)+abs(y)+abs(z).gt.1.5) then
               x=x-sign(1.,x)
               y=y-sign(1.,y)
               z=z-sign(1.,z)
            endif
            d=metric(1,1)*x*x+metric(2,2)*y*y+metric(3,3)*z*z
     -       +2.*(metric(1,2)*x*y+metric(1,3)*x*z+metric(2,3)*y*z)
            d=sqrt(d)
            do icc=1,ncoord
               if (d.lt.rcoord(icc)) then
                  if (type1.eq.typec(icc).and.type2.eq.typen(icc)) 
     -             nneigh(icc,j)=nneigh(icc,j)+1
                  if (type2.eq.typec(icc).and.type1.eq.typen(icc)) 
     -             nneigh(icc,i)=nneigh(icc,i)+1
               endif
            enddo
            ig=nint(d/dr)
            if (ig.le.nr) histogram(ig,ic)=histogram(ig,ic)+1
         enddo
      enddo
      return
      end







      subroutine rmc_gnorm(ntypes,ni,nxn,nr,dr,volume,gnorm)
      real gnorm(*)
      integer ni(*),nxn(*)
      parameter (pi=3.1415926)
      ic=1
      do itype=1,ntypes
         do jtype=itype,ntypes
            nxn(ic)=ni(itype)*ni(jtype)
            if (itype.ne.jtype) nxn(ic)=nxn(ic)*2
            ic=ic+1
         enddo
      enddo
      do ir=1,nr
         gnorm(ir)=(3.*ir*ir+0.25)*dr*dr*dr*2.*pi/(3.*volume)
      enddo
      return
      end



      subroutine rmc_sinqr(dr,nr,max_r,q,ifirst_q,ilast_q,rho,sqr)
      real q(*),sqr(max_r,*)
      parameter (pi=3.1415926)
*
*     Calculates a table of sin(qr) values
*
      iq=0
      do i=ifirst_q,ilast_q
         iq=iq+1
         do ir=1,nr
            r=real(ir)*dr
            sqr(ir,iq)=4.*pi*rho*r*sin(r*q(iq))/q(iq)*dr
         enddo
      enddo
      return
      end



      subroutine rmc_genmove(iseed,n,ncum,delta,metric,
     - imove,move)
      real    metric(3,3),move(6),delta(*)
      integer ncum(*) 
10    imove=int(ran(iseed)*n)+1
      mtype=1
      do while (imove.gt.ncum(mtype))
         mtype=mtype+1
      enddo
      if (delta(mtype).le.0.) goto 10
      move(4)=2.*(ran(iseed)-0.5)*delta(mtype)/sqrt(metric(1,1))
      move(5)=2.*(ran(iseed)-0.5)*delta(mtype)/sqrt(metric(2,2))
      move(6)=2.*(ran(iseed)-0.5)*delta(mtype)/sqrt(metric(3,3))
      return
      end



      subroutine rmc_newpos(move,imove,ifirst_atom,atoms,truncated)
      real move(6),atoms(3,*)
      logical truncated
*
      i=imove-ifirst_atom+1
      move(1)=atoms(1,i)
      move(2)=atoms(2,i)
      move(3)=atoms(3,i)
      move(4)=move(1)+move(4)+3.
      move(5)=move(2)+move(5)+3.
      move(6)=move(3)+move(6)+3.
      move(4)=2.*(move(4)/2.-int(move(4)/2.))-1.
      move(5)=2.*(move(5)/2.-int(move(5)/2.))-1.
      move(6)=2.*(move(6)/2.-int(move(6)/2.))-1.
      if (truncated.and.abs(move(4))+abs(move(5))+abs(move(6))
     - .gt.1.5) then
         move(4)=move(4)-sign(1.,move(4))
         move(5)=move(5)-sign(1.,move(5))
         move(6)=move(6)-sign(1.,move(6))
      endif
      return
      end




      subroutine rmc_changehist(ifirst_atom,ilast_atom,atoms,
     - ntypes,truncated,metric,ncum,dr,nr,max_r,rcut,imove,move,
     - dhistogram,max_cc,ncoord,
     - typec,typen,rcoord,dnneigh,nneighi,acceptable)
      real atoms(3,*),rcut(*),metric(3,3),move(6),rcoord(*)
      integer dhistogram(max_r,*),ncum(*)
      integer typec(*),typen(*),dnneigh(max_cc,*),nneighi(*)
      logical truncated,acceptable,inthen,innow
*
*     Calculates the change in the histogram and check whether the move
*     is acceptable according to the cut-off restrictions
*
      itype=1
      jtype=1
      do while (imove.gt.ncum(jtype))
         jtype=jtype+1
      enddo
      i=0
      do iatom=ifirst_atom,ilast_atom
         i=i+1
         if (iatom.gt.ncum(itype)) itype=itype+1
         if (iatom.ne.imove.and.acceptable) then
            i1=min(itype,jtype)
            j1=max(itype,jtype)
            ic=(i1-1)*(2*ntypes-i1)/2+j1
*
            x=atoms(1,i)-move(1)+3.
            y=atoms(2,i)-move(2)+3.
            z=atoms(3,i)-move(3)+3.
            x=2.*(x/2.-int(x/2.))-1.
            y=2.*(y/2.-int(y/2.))-1.
            z=2.*(z/2.-int(z/2.))-1.
            if (truncated.and.abs(x)+abs(y)+abs(z).gt.1.5) then
               x=x-sign(1.,x)
               y=y-sign(1.,y)
               z=z-sign(1.,z)
            endif
            dold=metric(1,1)*x*x+metric(2,2)*y*y+metric(3,3)*z*z
     -       +2.*(metric(1,2)*x*y+metric(1,3)*x*z+metric(2,3)*y*z)
            dold=sqrt(dold)
            ig=nint(dold/dr)
            if (ig.le.nr) dhistogram(ig,ic)=dhistogram(ig,ic)-1
*
            x=atoms(1,i)-move(4)+3.
            y=atoms(2,i)-move(5)+3.
            z=atoms(3,i)-move(6)+3.
            x=2.*(x/2.-int(x/2.))-1.
            y=2.*(y/2.-int(y/2.))-1.
            z=2.*(z/2.-int(z/2.))-1.
            if (truncated.and.abs(x)+abs(y)+abs(z).gt.1.5) then           
               x=x-sign(1.,x)
               y=y-sign(1.,y)
               z=z-sign(1.,z)
            endif
            dnew=metric(1,1)*x*x+metric(2,2)*y*y+metric(3,3)*z*z
     -       +2.*(metric(1,2)*x*y+metric(1,3)*x*z+metric(2,3)*y*z)
            dnew=sqrt(dnew)
            if (dnew.lt.rcut(ic).and.(dnew.lt.dold)) acceptable=.false.
            ig=nint(dnew/dr)
            if (ig.le.nr) dhistogram(ig,ic)=dhistogram(ig,ic)+1
*
            do icc=1,ncoord
               innow=dnew.lt.rcoord(icc)
               inthen=dold.lt.rcoord(icc)
               if (jtype.eq.typec(icc).and.itype.eq.typen(icc)) then
                  if (innow.and..not.inthen) nneighi(icc)=
     -             nneighi(icc)+1
                  if (inthen.and..not.innow) nneighi(icc)=
     -             nneighi(icc)-1
               endif
               if (jtype.eq.typen(icc).and.itype.eq.typec(icc)) then
                  if (innow.and..not.inthen) dnneigh(icc,i)=
     -             dnneigh(icc,i)+1
                  if (inthen.and..not.innow) dnneigh(icc,i)=
     -             dnneigh(icc,i)-1
               endif
            enddo
         endif
      enddo
      return
      end




      subroutine rmc_sumspar(max_r,max_q,max_expt,nexpt,npar,nq1,nq2,
     - ifirst_q,ilast_q,nr,gpar,sqr,coeff,sexpt,sum_s,sum_ss,sum_fs)
      real    sum_s(*),sum_ss(*),sum_fs(*)
      real    gpar(max_r,*),sqr(max_r,*),sexpt(max_q,nexpt)
      real    coeff(max_q,max_expt,*)
      real    spartial(100)
      integer nq1(*),nq2(*)
*
      do iexpt=1,nexpt
         sum_s(iexpt)=0.
         sum_ss(iexpt)=0.
         sum_fs(iexpt)=0.
      enddo
*
      i=0
      do iq=ifirst_q,ilast_q
         i=i+1
         do ic=1,npar
            spartial(ic)=0.
            do ir=1,nr
               spartial(ic)=spartial(ic)+(gpar(ir,ic)-1.)*
     -          sqr(ir,i)
            enddo
         enddo
         do iexpt=1,nexpt
            snew=0.
            do ic=1,npar
               snew=snew+coeff(i,iexpt,ic)*spartial(ic)
            enddo
            if (iq.ge.nq1(iexpt).and.iq.le.nq2(iexpt)) then
               sum_s(iexpt)=sum_s(iexpt)+snew
               sum_ss(iexpt)=sum_ss(iexpt)+snew*snew
               sum_fs(iexpt)=sum_fs(iexpt)+sexpt(i,iexpt)*snew
            endif
         enddo
      enddo
      return
      end




      subroutine rmc_sumstot(max_r,max_q,nexpt,nq1,nq2,
     - ifirst_q,ilast_q,nr,gtot,sqr,sexpt,sum_s,sum_ss,sum_fs)
      real    sum_s(*),sum_ss(*),sum_fs(*)
      real    sqr(max_r,*),sexpt(max_q,nexpt)
      real    gtot(max_r,*)
      integer nq1(*),nq2(*)
*
      do iexpt=1,nexpt
         sum_s(iexpt)=0.
         sum_ss(iexpt)=0.
         sum_fs(iexpt)=0.
         i=0
         do iq=ifirst_q,ilast_q
            i=i+1
            snew=0.
            do ir=1,nr
               snew=snew+gtot(ir,iexpt)*sqr(ir,i)
            enddo
            if (iq.ge.nq1(iexpt).and.iq.le.nq2(iexpt)) then
               sum_s(iexpt)=sum_s(iexpt)+snew
               sum_ss(iexpt)=sum_ss(iexpt)+snew*snew
               sum_fs(iexpt)=sum_fs(iexpt)+sexpt(i,iexpt)*snew
            endif
         enddo
      enddo
      return
      end


      subroutine rmc_sumg(max_r,max_q,nexpt,nq1,nq2,ifirst_q,ilast_q,
     - gtot,sexpt,sum_s,sum_ss,sum_fs)
      real    sum_s(*),sum_ss(*),sum_fs(*)
      real    sexpt(max_q,*)
      real    gtot(max_r,*)
      integer nq1(*),nq2(*)
*
      do iexpt=1,nexpt
         sum_s(iexpt)=0.
         sum_ss(iexpt)=0.
         sum_fs(iexpt)=0.
         i=0
         do iq=ifirst_q,ilast_q
            i=i+1
            if (iq.ge.nq1(iexpt).and.iq.le.nq2(iexpt)) then
               sum_s(iexpt)=sum_s(iexpt)+gtot(iq,iexpt)
               sum_ss(iexpt)=sum_ss(iexpt)+gtot(iq,iexpt)*gtot(iq,iexpt)
               sum_fs(iexpt)=sum_fs(iexpt)+sexpt(i,iexpt)*gtot(iq,iexpt)
            endif
         enddo
      enddo
      return
      end




      subroutine rmc_chisq(nexpt,nq1,nq2,renorm,offset,
     - factor,constant,sum_s,sum_fs,sum_ss,sum_f,sum_ff,
     - sigma,chisq,chisq1)
      real    sum_s(*),sum_ss(*),sum_fs(*),sum_f(*),sum_ff(*)
      real    factor(*),constant(*),sigma(*),chisq(*)
      integer nq1(*),nq2(*)
      logical renorm,offset
*
      do iexpt=1,nexpt
         qn=nq2(iexpt)-nq1(iexpt)+1
         if (renorm.and.offset) then
            factor(iexpt)=(qn*sum_fs(iexpt)
     -       -sum_f(iexpt)*sum_s(iexpt))/
     -       (qn*sum_ss(iexpt)-sum_s(iexpt)*sum_s(iexpt))
         elseif (renorm) then
            factor(iexpt)=sum_fs(iexpt)/sum_ss(iexpt)
         else
            factor(iexpt)=1.
         endif
         if (offset) then
            constant(iexpt)=(sum_f(iexpt)-factor(iexpt)
     -       *sum_s(iexpt))/qn
         else
            constant(iexpt)=0.
         endif
      enddo
*
      do iexpt=1,nexpt
         qn=nq2(iexpt)-nq1(iexpt)+1
         chisq(iexpt)=sum_ff(iexpt)+factor(iexpt)**2*sum_ss(iexpt)
     -    +constant(iexpt)**2*qn
     -    +2.*(constant(iexpt)*factor(iexpt)*sum_s(iexpt)
     -    -constant(iexpt)*sum_f(iexpt)
     -    -factor(iexpt)*sum_fs(iexpt))
         chisq(iexpt)=chisq(iexpt)/sigma(iexpt)/sigma(iexpt)
         chisq1=chisq1+chisq(iexpt)
      enddo
      return
      end


      subroutine rmc_coord(ni,ncum,ncoord,max_cc,typec,coordno,
     - nneigh,dnneigh,sigmac,cfnew,coordfrac,chisq1)
      integer ni(*),ncum(*),typec(*),coordno(*)
      integer nneigh(max_cc,*),dnneigh(max_cc,*)
      real    sigmac(*),cfnew(*),coordfrac(*)
*
      do icc=1,ncoord
         ncf=0
         do i=ncum(typec(icc))-ni(typec(icc))+1,ncum(typec(icc))
            if (nneigh(icc,i)+dnneigh(icc,i).eq.coordno(icc)) ncf=ncf+1
         enddo
         cfnew(icc)=real(ncf)/real(ni(typec(icc)))
         chisq1=chisq1+(cfnew(icc)-coordfrac(icc))**2/sigmac(icc)**2
      enddo
      return
      end









      subroutine rmc_makemove(atoms,n,imove,move,
     - nr,max_r,npar,histogram,dhistogram,ncoord,max_cc,nneigh,dnneigh)
      real    atoms(3,*),move(6)
      integer nneigh(max_cc,*),dnneigh(max_cc,*)
      integer histogram(max_r,*),dhistogram(max_r,*)
      atoms(1,imove)=move(4)
      atoms(2,imove)=move(5)
      atoms(3,imove)=move(6)
      do ic=1,npar
         do ir=1,nr
            histogram(ir,ic)=histogram(ir,ic)+dhistogram(ir,ic)
         enddo
      enddo
      do icc=1,ncoord
         do i=1,n
            nneigh(icc,i)=nneigh(icc,i)+dnneigh(icc,i)
         enddo
      enddo
      return
      end



